/****************************************************************************
** Meta object code from reading C++ file 'odfancurveqmlitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../corectrl-v1.5.2/src/core/components/controls/amd/fan/overdrive/curve/odfancurveqmlitem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'odfancurveqmlitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3AMD17OdFanCurveQMLItemE_t {};
} // unnamed namespace

template <> constexpr inline auto AMD::OdFanCurveQMLItem::qt_create_metaobjectdata<qt_meta_tag_ZN3AMD17OdFanCurveQMLItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AMD::OdFanCurveQMLItem",
        "curveChanged",
        "",
        "QVariantList",
        "curve",
        "curveRangeChanged",
        "tempMin",
        "tempMax",
        "speedMin",
        "speedMax",
        "stopAvailable",
        "stopTempAvailable",
        "stopChanged",
        "enabled",
        "stopTempChanged",
        "value",
        "updateCurvePoint",
        "QPointF",
        "oldPoint",
        "newPoint",
        "changeStop",
        "changeStopTemp",
        "minTemp",
        "maxTemp",
        "minSpeed",
        "maxSpeed",
        "stop",
        "stopTemp"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'curveChanged'
        QtMocHelpers::SignalData<void(QVariantList const &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'curveRangeChanged'
        QtMocHelpers::SignalData<void(qreal, qreal, qreal, qreal)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 6 }, { QMetaType::QReal, 7 }, { QMetaType::QReal, 8 }, { QMetaType::QReal, 9 },
        }}),
        // Signal 'stopAvailable'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stopTempAvailable'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stopChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'stopTempChanged'
        QtMocHelpers::SignalData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'updateCurvePoint'
        QtMocHelpers::SlotData<void(QPointF const &, QPointF const &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Slot 'changeStop'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'changeStopTemp'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'minTemp'
        QtMocHelpers::PropertyData<qreal>(22, QMetaType::QReal, QMC::DefaultPropertyFlags),
        // property 'maxTemp'
        QtMocHelpers::PropertyData<qreal>(23, QMetaType::QReal, QMC::DefaultPropertyFlags),
        // property 'minSpeed'
        QtMocHelpers::PropertyData<qreal>(24, QMetaType::QReal, QMC::DefaultPropertyFlags),
        // property 'maxSpeed'
        QtMocHelpers::PropertyData<qreal>(25, QMetaType::QReal, QMC::DefaultPropertyFlags),
        // property 'stop'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'stopTemp'
        QtMocHelpers::PropertyData<int>(27, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OdFanCurveQMLItem, qt_meta_tag_ZN3AMD17OdFanCurveQMLItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AMD::OdFanCurveQMLItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QMLItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3AMD17OdFanCurveQMLItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3AMD17OdFanCurveQMLItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3AMD17OdFanCurveQMLItemE_t>.metaTypes,
    nullptr
} };

void AMD::OdFanCurveQMLItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OdFanCurveQMLItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->curveChanged((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 1: _t->curveRangeChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[4]))); break;
        case 2: _t->stopAvailable(); break;
        case 3: _t->stopTempAvailable(); break;
        case 4: _t->stopChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->stopTempChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->updateCurvePoint((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 7: _t->changeStop((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->changeStopTemp((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OdFanCurveQMLItem::*)(QVariantList const & )>(_a, &OdFanCurveQMLItem::curveChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OdFanCurveQMLItem::*)(qreal , qreal , qreal , qreal )>(_a, &OdFanCurveQMLItem::curveRangeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OdFanCurveQMLItem::*)()>(_a, &OdFanCurveQMLItem::stopAvailable, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OdFanCurveQMLItem::*)()>(_a, &OdFanCurveQMLItem::stopTempAvailable, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OdFanCurveQMLItem::*)(bool )>(_a, &OdFanCurveQMLItem::stopChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (OdFanCurveQMLItem::*)(int )>(_a, &OdFanCurveQMLItem::stopTempChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->minTemp(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->maxTemp(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->minSpeed(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->maxSpeed(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->stop(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->stopTemp(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 4: _t->changeStop(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->changeStopTemp(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *AMD::OdFanCurveQMLItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AMD::OdFanCurveQMLItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3AMD17OdFanCurveQMLItemE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AMD::OdFanCurveProfilePart::Importer"))
        return static_cast< AMD::OdFanCurveProfilePart::Importer*>(this);
    if (!strcmp(_clname, "AMD::OdFanCurveProfilePart::Exporter"))
        return static_cast< AMD::OdFanCurveProfilePart::Exporter*>(this);
    return QMLItem::qt_metacast(_clname);
}

int AMD::OdFanCurveQMLItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMLItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void AMD::OdFanCurveQMLItem::curveChanged(QVariantList const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AMD::OdFanCurveQMLItem::curveRangeChanged(qreal _t1, qreal _t2, qreal _t3, qreal _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 2
void AMD::OdFanCurveQMLItem::stopAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void AMD::OdFanCurveQMLItem::stopTempAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void AMD::OdFanCurveQMLItem::stopChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void AMD::OdFanCurveQMLItem::stopTempChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
