// SPDX-License-Identifier: GPL-3.0-or-later
// Copyright 2019 Juan Palacios <jpalaciosdev@gmail.com>

#pragma once

#include <string_view>

namespace AMD::FanSpeedRPM {

static constexpr std::string_view ItemID{"AMD_FAN_SPEED_RPM"};

} // namespace AMD::FanSpeedRPM
