// This file is autogenerated. Changes will be overwritten.
#include "VJIZ3MDCXP/moc_app.cpp"
#include "VJIZ3MDCXP/moc_settings.cpp"
#include "VJIZ3MDCXP/moc_singleinstance.cpp"
#include "VJIZ3MDCXP/moc_systray.cpp"
#include "3QCLH6LIKH/moc_fanautoqmlitem.cpp"
#include "B4X2MGHYFV/moc_fancurveqmlitem.cpp"
#include "SRLUDDXB5V/moc_fanfixedqmlitem.cpp"
#include "WZOSBNM2YZ/moc_odfanautoqmlitem.cpp"
#include "TJXUAWDZKP/moc_odfancurveqmlitem.cpp"
#include "LWBRCZECPG/moc_pmdynamicfreqqmlitem.cpp"
#include "HFRVX77MXV/moc_pmfixedfreqqmlitem.cpp"
#include "H3D4S23V7W/moc_pmfreqodqmlitem.cpp"
#include "TNAYPLTE3C/moc_pmfreqoffsetqmlitem.cpp"
#include "XVO5WVJOYD/moc_pmfreqrangeqmlitem.cpp"
#include "LPBRWUPMAS/moc_pmfreqvoltqmlitem.cpp"
#include "4WZ5KKJ7B2/moc_pmvoltcurveqmlitem.cpp"
#include "ZECOBTEZQS/moc_pmvoltoffsetqmlitem.cpp"
#include "JCVFODWVZR/moc_pmpowercapqmlitem.cpp"
#include "L5KVWAM6IA/moc_pmpowerprofileqmlitem.cpp"
#include "SGZRUYFDQH/moc_pmautoqmlitem.cpp"
#include "OM7BTC23T2/moc_pmfixedqmlitem.cpp"
#include "GZTLT2RORB/moc_pmpowerstateqmlitem.cpp"
#include "RBHBQGMEPH/moc_controlgroupqmlitem.cpp"
#include "RBHBQGMEPH/moc_controlmodeqmlitem.cpp"
#include "4DEO27VBHI/moc_cpufreqqmlitem.cpp"
#include "RBHBQGMEPH/moc_noopqmlitem.cpp"
#include "IHZUJK2YFY/moc_cpuqmlitem.cpp"
#include "IHZUJK2YFY/moc_gpuqmlitem.cpp"
#include "VCA4543JQP/moc_graphitem.cpp"
#include "TAC5DWH4SE/moc_profilemanagerui.cpp"
#include "TAC5DWH4SE/moc_qmlitem.cpp"
#include "TAC5DWH4SE/moc_sysmodelqmlitem.cpp"
#include "TAC5DWH4SE/moc_systeminfoui.cpp"
#include "UDLBVYHCV3/moc_helpercontrol.cpp"
#include "UDLBVYHCV3/moc_helpermonitor.cpp"
