/****************************************************************************
** Meta object code from reading C++ file 'profilemanagerui.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../corectrl-v1.5.2/src/core/profilemanagerui.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'profilemanagerui.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16ProfileManagerUIE_t {};
} // unnamed namespace

template <> constexpr inline auto ProfileManagerUI::qt_create_metaobjectdata<qt_meta_tag_ZN16ProfileManagerUIE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ProfileManagerUI",
        "initProfiles",
        "",
        "QVariantList",
        "profiles",
        "profileAdded",
        "name",
        "exe",
        "icon",
        "hasCustomIcon",
        "isActive",
        "profileRemoved",
        "profileChanged",
        "profileActiveChanged",
        "active",
        "manualProfileToggled",
        "profileSaved",
        "profileInfoChanged",
        "oldName",
        "newName",
        "defaultIcon",
        "isProfileNameInUse",
        "profileName",
        "isExecutableNameInUse",
        "executableName",
        "isProfileUnsaved",
        "isProfileActive",
        "add",
        "base",
        "remove",
        "updateInfo",
        "activate",
        "toggleManualProfile",
        "loadSettings",
        "QUrl",
        "path",
        "applySettings",
        "resetSettings",
        "restoreSettings",
        "saveSettings",
        "exportProfile"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'initProfiles'
        QtMocHelpers::SignalData<void(QVariantList const &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'profileAdded'
        QtMocHelpers::SignalData<void(QString const &, QString const &, QString const &, bool, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Bool, 9 },
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'profileRemoved'
        QtMocHelpers::SignalData<void(QString const &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'profileChanged'
        QtMocHelpers::SignalData<void(QString const &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'profileActiveChanged'
        QtMocHelpers::SignalData<void(QString const &, bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Bool, 14 },
        }}),
        // Signal 'manualProfileToggled'
        QtMocHelpers::SignalData<void(QString const &, bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Bool, 14 },
        }}),
        // Signal 'profileSaved'
        QtMocHelpers::SignalData<void(QString const &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'profileInfoChanged'
        QtMocHelpers::SignalData<void(QString const &, QString const &, QString const &, QString const &, bool, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
            { QMetaType::Bool, 9 }, { QMetaType::Bool, 10 },
        }}),
        // Method 'defaultIcon'
        QtMocHelpers::MethodData<QString() const>(20, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'isProfileNameInUse'
        QtMocHelpers::MethodData<bool(QString const &)>(21, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 22 },
        }}),
        // Method 'isExecutableNameInUse'
        QtMocHelpers::MethodData<bool(QString const &)>(23, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 24 },
        }}),
        // Method 'isProfileUnsaved'
        QtMocHelpers::MethodData<bool(QString const &)>(25, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 22 },
        }}),
        // Method 'isProfileActive'
        QtMocHelpers::MethodData<bool(QString const &)>(26, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 22 },
        }}),
        // Method 'add'
        QtMocHelpers::MethodData<void(QString const &, QString const &, QString const &, QString const &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::QString, 28 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<void(QString const &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'updateInfo'
        QtMocHelpers::MethodData<void(QString const &, QString const &, QString const &, QString const &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
        // Method 'activate'
        QtMocHelpers::MethodData<void(QString const &, bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Bool, 14 },
        }}),
        // Method 'toggleManualProfile'
        QtMocHelpers::MethodData<void(QString const &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'loadSettings'
        QtMocHelpers::MethodData<void(QString const &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'loadSettings'
        QtMocHelpers::MethodData<bool(QString const &, QUrl const &)>(33, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 34, 35 },
        }}),
        // Method 'applySettings'
        QtMocHelpers::MethodData<void(QString const &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'resetSettings'
        QtMocHelpers::MethodData<void(QString const &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'restoreSettings'
        QtMocHelpers::MethodData<void(QString const &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'saveSettings'
        QtMocHelpers::MethodData<void(QString const &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'exportProfile'
        QtMocHelpers::MethodData<bool(QString const &, QUrl const &)>(40, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 34, 35 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProfileManagerUI, qt_meta_tag_ZN16ProfileManagerUIE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ProfileManagerUI::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ProfileManagerUIE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ProfileManagerUIE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16ProfileManagerUIE_t>.metaTypes,
    nullptr
} };

void ProfileManagerUI::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProfileManagerUI *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->initProfiles((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 1: _t->profileAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 2: _t->profileRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->profileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->profileActiveChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->manualProfileToggled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->profileSaved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->profileInfoChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 8: { QString _r = _t->defaultIcon();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->isProfileNameInUse((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->isExecutableNameInUse((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->isProfileUnsaved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->isProfileActive((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->add((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 14: _t->remove((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->updateInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 16: _t->activate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->toggleManualProfile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->loadSettings((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: { bool _r = _t->loadSettings((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->applySettings((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->resetSettings((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->restoreSettings((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->saveSettings((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: { bool _r = _t->exportProfile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ProfileManagerUI::*)(QVariantList const & )>(_a, &ProfileManagerUI::initProfiles, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProfileManagerUI::*)(QString const & , QString const & , QString const & , bool , bool )>(_a, &ProfileManagerUI::profileAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProfileManagerUI::*)(QString const & )>(_a, &ProfileManagerUI::profileRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProfileManagerUI::*)(QString const & )>(_a, &ProfileManagerUI::profileChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProfileManagerUI::*)(QString const & , bool )>(_a, &ProfileManagerUI::profileActiveChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProfileManagerUI::*)(QString const & , bool )>(_a, &ProfileManagerUI::manualProfileToggled, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProfileManagerUI::*)(QString const & )>(_a, &ProfileManagerUI::profileSaved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProfileManagerUI::*)(QString const & , QString const & , QString const & , QString const & , bool , bool )>(_a, &ProfileManagerUI::profileInfoChanged, 7))
            return;
    }
}

const QMetaObject *ProfileManagerUI::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ProfileManagerUI::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ProfileManagerUIE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ProfileManagerUI::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void ProfileManagerUI::initProfiles(QVariantList const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ProfileManagerUI::profileAdded(QString const & _t1, QString const & _t2, QString const & _t3, bool _t4, bool _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 2
void ProfileManagerUI::profileRemoved(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ProfileManagerUI::profileChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ProfileManagerUI::profileActiveChanged(QString const & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void ProfileManagerUI::manualProfileToggled(QString const & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void ProfileManagerUI::profileSaved(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void ProfileManagerUI::profileInfoChanged(QString const & _t1, QString const & _t2, QString const & _t3, QString const & _t4, bool _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}
QT_WARNING_POP
