/****************************************************************************
** Meta object code from reading C++ file 'pmfreqrangeqmlitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../corectrl-v1.5.2/src/core/components/controls/amd/pm/advanced/overdrive/freqrange/pmfreqrangeqmlitem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pmfreqrangeqmlitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3AMD18PMFreqRangeQMLItemE_t {};
} // unnamed namespace

template <> constexpr inline auto AMD::PMFreqRangeQMLItem::qt_create_metaobjectdata<qt_meta_tag_ZN3AMD18PMFreqRangeQMLItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AMD::PMFreqRangeQMLItem",
        "controlLabelChanged",
        "",
        "label",
        "stateRangeChanged",
        "min",
        "max",
        "statesChanged",
        "QVariantList",
        "states",
        "someStateChanged",
        "index",
        "freq",
        "changeState",
        "instanceID"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'controlLabelChanged'
        QtMocHelpers::SignalData<void(QString const &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'stateRangeChanged'
        QtMocHelpers::SignalData<void(int, int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'statesChanged'
        QtMocHelpers::SignalData<void(QVariantList const &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'someStateChanged'
        QtMocHelpers::SignalData<void(int, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'changeState'
        QtMocHelpers::SlotData<void(int, int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'instanceID'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PMFreqRangeQMLItem, qt_meta_tag_ZN3AMD18PMFreqRangeQMLItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AMD::PMFreqRangeQMLItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QMLItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3AMD18PMFreqRangeQMLItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3AMD18PMFreqRangeQMLItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3AMD18PMFreqRangeQMLItemE_t>.metaTypes,
    nullptr
} };

void AMD::PMFreqRangeQMLItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PMFreqRangeQMLItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->controlLabelChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->stateRangeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->statesChanged((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 3: _t->someStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->changeState((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PMFreqRangeQMLItem::*)(QString const & )>(_a, &PMFreqRangeQMLItem::controlLabelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PMFreqRangeQMLItem::*)(int , int )>(_a, &PMFreqRangeQMLItem::stateRangeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PMFreqRangeQMLItem::*)(QVariantList const & )>(_a, &PMFreqRangeQMLItem::statesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PMFreqRangeQMLItem::*)(int , int )>(_a, &PMFreqRangeQMLItem::someStateChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->instanceID(); break;
        default: break;
        }
    }
}

const QMetaObject *AMD::PMFreqRangeQMLItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AMD::PMFreqRangeQMLItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3AMD18PMFreqRangeQMLItemE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AMD::PMFreqRangeProfilePart::Importer"))
        return static_cast< AMD::PMFreqRangeProfilePart::Importer*>(this);
    if (!strcmp(_clname, "AMD::PMFreqRangeProfilePart::Exporter"))
        return static_cast< AMD::PMFreqRangeProfilePart::Exporter*>(this);
    return QMLItem::qt_metacast(_clname);
}

int AMD::PMFreqRangeQMLItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMLItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void AMD::PMFreqRangeQMLItem::controlLabelChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AMD::PMFreqRangeQMLItem::stateRangeChanged(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void AMD::PMFreqRangeQMLItem::statesChanged(QVariantList const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void AMD::PMFreqRangeQMLItem::someStateChanged(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
